@@ -1,5 +1,6 @@
 from __future__ import unicode_literals
 
+import logging
 import re
 from collections import defaultdict, deque
 
@@ -11,13 +12,15 @@ from six.moves.urllib.parse import urljoin
 from rbtools.api.cache import MINIMUM_VERSION
 from rbtools.api.decorators import request_method_decorator
 from rbtools.api.request import HttpRequest
+from rbtools.api.utils import rem_mime_format
 from rbtools.utils.graphs import path_exists
 
 
 RESOURCE_MAP = {}
 LINKS_TOK = 'links'
-LINK_KEYS = set(['href', 'method', 'title'])
-_EXCLUDE_ATTRS = [LINKS_TOK, 'stat']
+EXPANDED_TOKEN = '_expanded'
+LINK_KEYS = set(['href', 'method', 'title', 'mimetype'])
+_EXCLUDE_ATTRS = [LINKS_TOK, EXPANDED_TOKEN, 'stat']
 _EXTRA_DATA_PREFIX = 'extra_data__'
 
 
@@ -48,6 +51,42 @@ def _preprocess_fields(fields):
         yield name, value
 
 
+def _create_resource_for_field(parent_resource, field_payload,
+                               mimetype, item_mimetype=None, url=None):
+    """Create a resource instance based on field data.
+
+    This will construct a resource instance for the payload of a field,
+    using the given mimetype to identify it. This is intended for use with
+    expanded resources or items in lists.
+
+    Args:
+        parent_resource (Resource):
+            The resource containing the field payload.
+
+        field_payload (dict):
+            The field payload to use as the new resource's payload.
+
+        mimetype (unicode):
+            The mimetype of the resource.
+
+        item_mimetype (unicode, optional):
+            The mimetype of any items in the resource, if this resource
+            represents a list.
+
+        url (unicode, optional):
+            The URL of the resource, if one is available.
+    """
+    # We need to import this here to avoid circular imports.
+    from rbtools.api.factory import create_resource
+
+    return create_resource(transport=parent_resource._transport,
+                           payload=field_payload,
+                           url=url,
+                           mime_type=mimetype,
+                           item_mime_type=item_mimetype,
+                           guess_token=False)
+
+
 @request_method_decorator
 def _create(resource, data=None, query_args={}, *args, **kwargs):
     """Generate a POST request on a resource.
@@ -157,7 +196,7 @@ class Resource(object):
         self._transport = transport
         self._token = token
         self._payload = payload
-        self._excluded_attrs = self._excluded_attrs + _EXCLUDE_ATTRS
+        self._excluded_attrs = set(self._excluded_attrs + _EXCLUDE_ATTRS)
 
         # Determine where the links live in the payload. This
         # can either be at the root, or inside the resources
@@ -171,6 +210,17 @@ class Resource(object):
             self._payload[LINKS_TOK] = {}
             self._links = {}
 
+        # If we've expanded any fields, we'll try to convert the expanded
+        # payloads into resources. We can only do this if talking to
+        # Review Board 4.0+.
+        if EXPANDED_TOKEN in self._payload:
+            self._expanded_info = self._payload[EXPANDED_TOKEN]
+        elif (token and isinstance(self._payload[token], dict) and
+              EXPANDED_TOKEN in self._payload[token]):
+            self._expanded_info = self._payload[token][EXPANDED_TOKEN]
+        else:
+            self._expanded_info = {}
+
         # Add a method for each supported REST operation, and
         # for retrieving 'self'.
         for link, method in six.iteritems(SPECIAL_LINKS):
@@ -189,19 +239,77 @@ class Resource(object):
                         lambda resource=self, url=body['href'], **kwargs: (
                             self._get_url(url, **kwargs)))
 
-    def _wrap_field(self, field):
-        if isinstance(field, dict):
-            dict_keys = set(field.keys())
+    def _wrap_field(self, field_payload, field_url=None, field_mimetype=None,
+                    list_item_mimetype=None, force_resource=False):
+        """Wrap the value of a field in a resource or field object.
 
-            if ('href' in dict_keys and
-                len(dict_keys.difference(LINK_KEYS)) == 0):
-                return ResourceLinkField(self, field)
+        This determines a suitable wrapper for a field, turning it into
+        a resource or a wrapper with utility methods that can be used to
+        interact with the field or perform additional queries.
+
+        Args:
+            field_payload (object):
+                The payload of the field. The type of value determines the
+                way in which this is wrapped.
+
+            field_url (unicode, optional):
+                The URL representing the payload in the field, if one is
+                available. If not provided, one may be computed, depending
+                on the type and contents of the field.
+
+            field_mimetype (unicode, optional):
+                The mimetype used to represent the field. If provided, this
+                may result in the wrapper being a :py:class:`Resource`
+                subclass.
+
+            list_item_mimetype (unicode, optional):
+                The mimetype used or any items within the field, if the
+                field is a list.
+
+            force_resource (bool, optional):
+                Force the return of a resource, even a generic one, instead
+                of a field wrapper.
+
+        Returns:
+            object:
+            A wrapper, or the field payload. This may be one of:
+
+            1. A subclass of :py:class:`Resource`.
+            2. A field wrapper (:py:class:`ResourceDictField`,
+               :py:class:`ResourceLinkField`, or
+               :py:class:`ResourceListField`).
+            3. The field payload itself, if no wrapper is needed.
+        """
+        if isinstance(field_payload, dict):
+            if (force_resource or
+                (field_mimetype and
+                 rem_mime_format(field_mimetype) in RESOURCE_MAP)):
+                # We have a resource backing this mimetype. Try to create
+                # an instance of the resource for this payload.
+                if not field_url:
+                    try:
+                        field_url = field_payload['links']['self']['href']
+                    except KeyError:
+                        field_url = ''
+
+                return _create_resource_for_field(parent_resource=self,
+                                                  field_payload=field_payload,
+                                                  mimetype=field_mimetype,
+                                                  url=field_url)
             else:
-                return ResourceDictField(self, field)
-        elif isinstance(field, list):
-            return ResourceListField(self, field)
+                # If the payload consists solely of link-supported keys,
+                # then we'll return a special ResourceLinkField. Anything
+                # else is treated as a standard dictionary.
+                if ('href' in field_payload and
+                    not set(field_payload.keys()) - LINK_KEYS):
+                    return ResourceLinkField(self, field_payload)
+                else:
+                    return ResourceDictField(self, field_payload)
+        elif isinstance(field_payload, list):
+            return ResourceListField(self, field_payload,
+                                     item_mimetype=list_item_mimetype)
         else:
-            return field
+            return field_payload
 
     @property
     def links(self):
@@ -292,21 +400,27 @@ class ResourceListField(list):
 
     Acts as a normal list, but wraps any returned items.
     """
-    def __init__(self, resource, list_field):
+    def __init__(self, resource, list_field, item_mimetype=None):
         super(ResourceListField, self).__init__(list_field)
+
         self._resource = resource
+        self._item_mimetype = item_mimetype
 
     def __getitem__(self, key):
         item = super(ResourceListField, self).__getitem__(key)
-        return self._resource._wrap_field(item)
+        return self._resource._wrap_field(item,
+                                          field_mimetype=self._item_mimetype)
 
     def __iter__(self):
         for item in super(ResourceListField, self).__iter__():
-            yield self._resource._wrap_field(item)
+            yield self._resource._wrap_field(
+                item,
+                field_mimetype=self._item_mimetype)
 
     def __repr__(self):
-        return '%s(resource=%r, list_field=%s)' % (
+        return '%s(resource=%r, item_mimetype=%s, list_field=%s)' % (
             self.__class__.__name__,
+            self._item_mimetype,
             self._resource,
             super(ResourceListField, self).__repr__())
 
@@ -341,12 +455,51 @@ class ItemResource(Resource):
                 self._fields[name] = value
 
     def __getattr__(self, name):
-        if name in self._fields:
-            return self._wrap_field(self._fields[name])
-        else:
+        """Return the value for an attribute on the resource.
+
+        If the attribute represents an expanded resource, and there's
+        information available on the expansion (available in Review Board
+        4.0+), then a resource instance will be returned.
+
+        If the attribute otherwise represents a dictionary, list, or a link,
+        a wrapper may be returned.
+
+        Args:
+            name (str):
+                The name of the attribute.
+
+        Returns:
+            object:
+            The attribute value, or a wrapper or resource representing that
+            value.
+
+        Raises:
+            AttributeError:
+                A field with the given attribute name was not found.
+        """
+        try:
+            field_payload = self._fields[name]
+        except KeyError:
             raise AttributeError('This %s does not have an attribute "%s".'
                                  % (self.__class__.__name__, name))
 
+        expand_info = self._expanded_info.get(name, {})
+
+        if isinstance(field_payload, dict):
+            value = self._wrap_field(
+                field_payload=field_payload,
+                field_mimetype=expand_info.get('item_mimetype'))
+        elif isinstance(field_payload, list):
+            value = self._wrap_field(
+                field_payload=field_payload,
+                field_url=expand_info.get('list_url'),
+                field_mimetype=expand_info.get('list_mimetype'),
+                list_item_mimetype=expand_info.get('item_mimetype'))
+        else:
+            value = self._wrap_field(field_payload)
+
+        return value
+
     def __getitem__(self, key):
         try:
             return self.__getattr__(key)
@@ -361,8 +514,14 @@ class ItemResource(Resource):
             yield key
 
     def iteritems(self):
-        for key, value in six.iteritems(self._fields):
-            yield (key, self._wrap_field(value))
+        """Iterate through all field/value pairs in the resource.
+
+        Yields:
+            tuple:
+            A tuple in ``(field_name, value)`` form.
+        """
+        for key in self.iterfields():
+            yield key, self.__getattr__(key)
 
     def __repr__(self):
         return '%s(transport=%r, payload=%r, url=%r, token=%r)' % (
@@ -440,24 +599,24 @@ class ListResource(Resource):
     def __bool__(self):
         return True
 
-    def __getitem__(self, key):
-        payload = self._item_list[key]
+    def __getitem__(self, index):
+        """Return the item at the specified index.
 
-        # TODO: Should try and guess the url based on the parent url,
-        # and the id number if the self link doesn't exist.
-        try:
-            url = payload['links']['self']['href']
-        except KeyError:
-            url = ''
+        Args:
+            index (int):
+                The index of the item to retrieve.
 
-        # We need to import this here because of the mutual imports.
-        from rbtools.api.factory import create_resource
+        Returns:
+            object:
+            The item at the specified index.
 
-        return create_resource(self._transport,
-                               payload,
-                               url,
-                               mime_type=self._item_mime_type,
-                               guess_token=False)
+        Raises:
+            IndexError:
+                The index is out of range.
+        """
+        return self._wrap_field(self._item_list[index],
+                                field_mimetype=self._item_mime_type,
+                                force_resource=True)
 
     def __iter__(self):
         for i in range(self.num_items):
@@ -570,6 +729,127 @@ class RootResource(ItemResource):
         return HttpRequest(url, query_args=kwargs)
 
 
+@resource_mimetype('application/vnd.reviewboard.org.commit')
+class DiffCommitItemResource(ItemResource):
+    """The commit resource-specific class."""
+
+    @request_method_decorator
+    def get_patch(self, **kwargs):
+        """Retrieve the actual diff file contents.
+
+        Args:
+            **kwargs (dict):
+                Query args to pass to
+                :py:meth:`~rbtools.api.request.HttpRequest.__init__`.
+
+        Returns:
+            ItemResource:
+            A resource payload whose :py:attr:`~ItemResource.data` attribute is
+            the requested patch.
+        """
+        return HttpRequest(self._url, query_args=kwargs, headers={
+            'Accept': 'text/x-patch',
+        })
+
+
+@resource_mimetype('application/vnd.reviewboard.org.draft-commit')
+class DraftDiffCommitItemResource(ItemResource):
+    """The draft commit resource-specific class."""
+    pass
+
+
+@resource_mimetype('application/vnd.reviewboard.org.draft-commits')
+class DraftDiffCommitListResource(ListResource):
+    """The draft commit list resource-specific class.
+
+    Provides additional functionality in the uploading of new commits.
+    """
+
+    @request_method_decorator
+    def upload_commit(self, validation_info, diff, commit_id, parent_id,
+                      author_name, author_email, author_date, commit_message,
+                      committer_name=None, committer_email=None,
+                      committer_date=None, parent_diff=None, **kwargs):
+        """Upload a commit.
+
+        Args:
+            validation_info (unicode):
+                The validation info, or ``None`` if this is the first commit in
+                a series.
+
+            diff (bytes):
+                The diff contents.
+
+            commit_id (unicode):
+                The ID of the commit being uploaded.
+
+            parent_id (unicode):
+                The ID of the parent commit.
+
+            author_name (unicode):
+                The name of the author.
+
+            author_email (unicode):
+                The e-mail address of the author.
+
+            author_date (unicode):
+                The date and time the commit was authored in ISO 8601 format.
+
+            committer_name (unicode, optional):
+                The name of the committer (if applicable).
+
+            committer_email (unicode, optional):
+                The e-mail address of the committer (if applicable).
+
+            committer_date (unicode, optional):
+                The date and time the commit was committed in ISO 8601 format
+                (if applicable).
+
+            parent_diff (bytes, optional):
+                The contents of the parent diff.
+
+            **kwargs (dict):
+                Keyword argument used to build the querystring for the request
+                URL.
+
+        Returns:
+            DraftDiffCommitItemResource:
+            The created resource.
+
+        Raises:
+            rbtools.api.errors.APIError:
+                An error occurred while uploading the commit.
+        """
+        request = HttpRequest(self._url, method='POST', query_args=kwargs)
+
+        request.add_file('diff', 'diff', diff)
+        request.add_field('commit_id', commit_id)
+        request.add_field('parent_id', parent_id)
+        request.add_field('commit_message', commit_message)
+        request.add_field('author_name', author_name)
+        request.add_field('author_email', author_email)
+        request.add_field('author_date', author_date)
+
+        if validation_info:
+            request.add_field('validation_info', validation_info)
+
+        if committer_name and committer_email and committer_date:
+            request.add_field('committer_name', committer_name)
+            request.add_field('committer_email', committer_email)
+            request.add_field('committer_date', committer_date)
+        elif committer_name or committer_email or committer_name:
+            logging.warning(
+                'Either all or none of committer_name, committer_email, and '
+                'committer_date must be provided to upload_commit. None of '
+                'these fields will be submitted.'
+            )
+
+        if parent_diff:
+            request.add_file('parent_diff', 'parent_diff', parent_diff)
+
+        return request
+
+
 class DiffUploaderMixin(object):
     """A mixin for uploading diffs to a resource."""
 
@@ -619,6 +899,28 @@ class DiffListResource(DiffUploaderMixin, ListResource):
             base_commit_id=base_commit_id,
             **kwargs)
 
+    @request_method_decorator
+    def create_empty(self, base_commit_id=None, **kwargs):
+        """Create an empty DiffSet that commits can be added to.
+
+        Args:
+            base_commit_id (unicode, optional):
+                The base commit ID of the diff.
+
+            **kwargs (dict):
+                Keyword arguments to encode into the querystring of the request
+                URL.
+        Returns:
+            DiffItemResource:
+            The created resource.
+        """
+        request = HttpRequest(self._url, method='POST', query_args=kwargs)
+
+        if base_commit_id:
+            request.add_field('base_commit_id', base_commit_id)
+
+        return request
+
 
 @resource_mimetype('application/vnd.reviewboard.org.diff')
 class DiffResource(ItemResource):
@@ -634,6 +936,50 @@ class DiffResource(ItemResource):
             'Accept': 'text/x-patch',
         })
 
+    @request_method_decorator
+    def finalize_commit_series(self, cumulative_diff, validation_info,
+                               parent_diff=None):
+        """Finalize a commit series.
+
+        Args:
+            cumulative_diff (bytes):
+                The cumulative diff of the entire commit series.
+
+            validation_info (unicode):
+                The validation information returned by validatin the last
+                commit in the series with the
+                :py:class:`ValidateDiffCommitResource`.
+
+            parent_diff (bytes, optional):
+                An optional parent diff.
+
+                This will be the same parent diff uploaded with each commit.
+
+        Returns:
+            DiffItemResource:
+            The finalized diff resource.
+        """
+        if not isinstance(cumulative_diff, bytes):
+            raise TypeError('cumulative_diff must be byte string, not %s'
+                            % type(cumulative_diff))
+
+        if parent_diff is not None and not isinstance(parent_diff, bytes):
+            raise TypeError('parent_diff must be byte string, not %s'
+                            % type(cumulative_diff))
+
+        request = HttpRequest(self.links['self']['href'],
+                              method='PUT')
+
+        request.add_field('finalize_commit_series', True)
+        request.add_file('cumulative_diff', 'cumulative_diff',
+                         cumulative_diff)
+        request.add_field('validation_info', validation_info)
+
+        if parent_diff is not None:
+            request.add_file('parent_diff', 'parent_diff', parent_diff)
+
+        return request
+
 
 @resource_mimetype('application/vnd.reviewboard.org.file')
 class FileDiffResource(ItemResource):
@@ -851,3 +1197,60 @@ class ValidateDiffResource(DiffUploaderMixin, ItemResource):
             request.add_field('base_commit_id', base_commit_id)
 
         return request
+
+
+@resource_mimetype('application/vnd.reviewboard.org.commit-validation')
+class ValidateDiffCommitResource(ItemResource):
+    """The commit validation resource specific base class."""
+
+    @request_method_decorator
+    def validate_commit(self, repository, diff, commit_id, parent_id,
+                        parent_diff=None, base_commit_id=None,
+                        validation_info=None, **kwargs):
+        """Validate the diff for a commit.
+
+        Args:
+            repository (unicode):
+                The name of the repository.
+
+            diff (bytes):
+                The contents of the diff to validate.
+
+            commit_id (unicode):
+                The ID of the commit being validated.
+
+            parent_id (unicode):
+                The ID of the parent commit.
+
+            parent_diff (bytes, optional):
+                The contents of the parent diff.
+
+            base_commit_id (unicode, optional):
+                The base commit ID.
+
+            validation_info (unicode, optional):
+                Validation information from a previous call to this resource.
+
+            **kwargs (dict):
+                Keyword arguments used to build the querystring.
+
+        Returns:
+            ValidateDiffCommitResource:
+            The validation result.
+        """
+        request = HttpRequest(self._url, method='POST', query_args=kwargs)
+        request.add_file('diff', 'diff', diff)
+        request.add_field('repository', repository)
+        request.add_field('commit_id', commit_id)
+        request.add_field('parent_id', parent_id)
+
+        if parent_diff:
+            request.add_file('parent_diff', 'parent_diff', parent_diff)
+
+        if base_commit_id:
+            request.add_field('base_commit_id', base_commit_id)
+
+        if validation_info:
+            request.add_field('validation_info', validation_info)
+
+        return request
