@@ -18,6 +18,34 @@ from rbtools.utils.process import execute
 SCMCLIENTS = None
 
 
+class PatchAuthor(object):
+    """The author of a patch or commit.
+
+    This wraps the full name and e-mail address of a commit or patch's
+    author primarily for use in :py:meth:`SCMClient.apply_patch`.
+
+    Attributes:
+        fullname (unicode):
+            The full name of the author.
+
+        email (unicode):
+            The e-mail address of the author.
+    """
+
+    def __init__(self, full_name, email):
+        """Initialize the author information.
+
+        Args:
+            full_name (unicode):
+                The full name of the author.
+
+            email (unicode):
+                The e-mail address of the author.
+        """
+        self.fullname = full_name
+        self.email = email
+
+
 class PatchResult(object):
     """The result of a patch operation.
 
@@ -58,6 +86,8 @@ class SCMClient(object):
 
     name = None
 
+    #: Whether or not the SCM client can generate a commit history.
+    supports_commit_history = False
     supports_diff_extra_args = False
     supports_diff_exclude_patterns = False
     supports_no_renames = False
@@ -253,6 +283,22 @@ class SCMClient(object):
             'base_commit_id': None,
         }
 
+    def get_commit_history(self, revisions):
+        """Return the commit history between the given revisions.
+
+        Derived classes must override this method if they support posting with
+        history.
+
+        Args:
+            revisions (dict):
+                The parsed revision spec to use to generate the history.
+
+        Returns:
+            list of dict:
+            The history entries.
+        """
+        raise NotImplementedError
+
     def _get_server_from_config(self, config, repository_info):
         """Return the Review Board server URL in the config.
 
@@ -657,13 +703,14 @@ class SCMClient(object):
 class RepositoryInfo(object):
     """A representation of a source code repository."""
 
-    def __init__(self, path=None, base_path=None, local_path=None,
+    def __init__(self, path=None, base_path=None, local_path=None, name=None,
                  supports_changesets=False, supports_parent_diffs=False):
         """Initialize the object.
 
         Args:
-            path (unicode, optional):
-                The path of the repository.
+            path (unicode or list of unicode, optional):
+                The path of the repository, or a list of possible paths
+                (with the primary one appearing first).
 
             base_path (unicode, optional):
                 The relative path between the current working directory and the
@@ -674,6 +721,10 @@ class RepositoryInfo(object):
                 sometimes be the same as ``path``, but may not be (since that
                 can contain a remote repository path).
 
+            name (unicode, optional):
+                The name of the repository, as configured on Review Board.
+                This might be available through some repository metadata.
+
             supports_changesets (bool, optional):
                 Whether the repository type supports changesets that store
                 their data server-side.
@@ -685,6 +736,7 @@ class RepositoryInfo(object):
         self.path = path
         self.base_path = base_path
         self.local_path = local_path
+        self.name = name
         self.supports_changesets = supports_changesets
         self.supports_parent_diffs = supports_parent_diffs
         logging.debug('Repository info: %s', self)
@@ -754,7 +806,8 @@ def load_scmclients(config, options):
             logging.exception('Could not load SCM Client "%s"', ep.name)
 
 
-def scan_usable_client(config, options, client_name=None):
+def scan_usable_client(config, options, client_name=None,
+                       require_repository_info=True):
     """Scan for a usable SCMClient.
 
     Args:
@@ -769,6 +822,11 @@ def scan_usable_client(config, options, client_name=None):
             can be used to disambiguate if there are nested repositories, or to
             speed up detection.
 
+        require_repository_info (bool, optional):
+            Whether information on a repository is required. This is the
+            default. If disabled, this will return ``None`` for the repository
+            information if a matching repository could not be found.
+
     Returns:
         tuple:
         A 2-tuple, containing the repository info structure and the tool
@@ -837,7 +895,28 @@ def scan_usable_client(config, options, client_name=None):
                 # posting against a remote SVN repository there will be no
                 # local path), just default to the first repository found
                 repository_info, tool = candidate_repos[0]
-    else:
+
+    if repository_info is not None:
+        # Verify that options specific to an SCM Client have not been mis-used.
+        if (getattr(options, 'change_only', False) and
+            not repository_info.supports_changesets):
+            logging.error('The --change-only option is not valid for the '
+                          'current SCM client.\n')
+            sys.exit(1)
+
+        if (getattr(options, 'parent_branch', None) and
+            not repository_info.supports_parent_diffs):
+            logging.error('The --parent option is not valid for the '
+                          'current SCM client.')
+            sys.exit(1)
+
+        if (not isinstance(tool, PerforceClient) and
+            (getattr(options, 'p4_client', None) or
+             getattr(options, 'p4_port', None))):
+            logging.error('The --p4-client and --p4-port options are not '
+                          'valid for the current SCM client.\n')
+            sys.exit(1)
+    elif require_repository_info:
         if client_name:
             logging.error('The provided repository type was not detected '
                           'in the current directory.')
@@ -850,27 +929,7 @@ def scan_usable_client(config, options, client_name=None):
 
         sys.exit(1)
 
-    # Verify that options specific to an SCM Client have not been mis-used.
-    if (getattr(options, 'change_only', False) and
-        not repository_info.supports_changesets):
-        sys.stderr.write('The --change-only option is not valid for the '
-                         'current SCM client.\n')
-        sys.exit(1)
-
-    if (getattr(options, 'parent_branch', None) and
-        not repository_info.supports_parent_diffs):
-        sys.stderr.write('The --parent option is not valid for the '
-                         'current SCM client.\n')
-        sys.exit(1)
-
-    if (not isinstance(tool, PerforceClient) and
-        (getattr(options, 'p4_client', None) or
-         getattr(options, 'p4_port', None))):
-        sys.stderr.write('The --p4-client and --p4-port options are not valid '
-                         'for the current SCM client.\n')
-        sys.exit(1)
-
-    return (repository_info, tool)
+    return repository_info, tool
 
 
 def print_clients(config, options):
