@@ -8,6 +8,7 @@ import re
 import sys
 
 import six
+from six.moves import zip
 
 from rbtools.clients import PatchResult, SCMClient, RepositoryInfo
 from rbtools.clients.errors import (AmendError,
@@ -36,9 +37,12 @@ class GitClient(SCMClient):
     """
 
     name = 'Git'
+
+    supports_commit_history = True
     supports_diff_exclude_patterns = True
     supports_no_renames = True
     supports_patch_revert = True
+
     can_amend_commit = True
     can_merge = True
     can_push_upstream = True
@@ -50,6 +54,9 @@ class GitClient(SCMClient):
     TYPE_GIT_SVN = 1
     TYPE_GIT_P4 = 2
 
+    _NUL = '\x00'
+    _FIELD_SEP = '\x1f'
+
     def __init__(self, **kwargs):
         """Initialize the client.
 
@@ -253,10 +260,9 @@ class GitClient(SCMClient):
                               '--help": skipping Git')
                 return None
 
-        git_dir = self._execute([self.git, 'rev-parse', '--git-dir'],
-                                ignore_errors=True).rstrip('\n')
+        git_dir = self._get_git_dir()
 
-        if git_dir.startswith('fatal:') or not os.path.isdir(git_dir):
+        if git_dir is None:
             return None
 
         # Sometimes core.bare is not set, and generates an error, so ignore
@@ -388,6 +394,32 @@ class GitClient(SCMClient):
                                   supports_parent_diffs=True)
         return None
 
+    def _get_git_dir(self):
+        """Return the current git directory.
+
+        This will return the :file:`.git` directory corresponding to the full
+        checkout, traversing up in the case of worktrees.
+
+        Returns:
+            unicode:
+            The path to the :file:`.git` directory for the repository.
+        """
+        git_dir = self._execute([self.git, 'rev-parse', '--git-dir'],
+                                ignore_errors=True).rstrip('\n')
+
+        if git_dir.startswith('fatal:') or not os.path.isdir(git_dir):
+            return None
+
+        try:
+            # In the case of a worktree, find the common gitdir.
+            with open(os.path.join(git_dir, 'commondir')) as f:
+                common_dir = f.read().strip()
+                git_dir = os.path.abspath(os.path.join(git_dir, common_dir))
+        except IOError:
+            pass
+
+        return git_dir
+
     def _strip_heads_prefix(self, ref):
         """Strip the heads prefix off of a reference name.
 
@@ -512,6 +544,22 @@ class GitClient(SCMClient):
                 if remote and remote != '.' and merge:
                     return '%s/%s' % (remote, merge)
 
+            # As of Git 2.28, users can configure a default main branch name.
+            # In most cases, this will be handled by the _get_remote call
+            # above. This here is a fallback to a fallback, and assumes that if
+            # they're operating with a bare checkout with a non-standard main
+            # branch name, they're configured correctly.
+            defaultBranch = self._execute([self.git, 'config', '--global',
+                                           '--get', 'init.defaultBranch'],
+                                          ignore_errors=True).strip()
+
+            if (defaultBranch and
+                os.path.exists(os.path.join(self._get_git_dir(), 'refs',
+                                            'remotes', 'origin',
+                                            defaultBranch))):
+                return 'origin/%s' % defaultBranch
+
+            # Finally, just fall back to the old standard.
             return 'origin/master'
         else:
             raise ValueError('Unknown git client type %s' % self._type)
@@ -579,7 +627,8 @@ class GitClient(SCMClient):
         return youngest_remote_commit
 
     def diff(self, revisions, include_files=[], exclude_patterns=[],
-             no_renames=False, extra_args=[]):
+             no_renames=False, extra_args=[], with_parent_diff=True,
+             git_find_renames_threshold=None, **kwargs):
         """Perform a diff using the given revisions.
 
         If no revisions are specified, this will do a diff of the contents of
@@ -604,10 +653,22 @@ class GitClient(SCMClient):
                 A list of shell-style glob patterns to blacklist during diff
                 generation.
 
+            no_renames (bool, optional):
+                Whether to avoid rename detection.
+
             extra_args (list, unused):
                 Additional arguments to be passed to the diff generation.
                 Unused for git.
 
+            with_parent_diff (bool, optional):
+                Whether or not to compute a parent diff.
+
+            git_find_renames_threshold (unicode, optional):
+                The threshold to pass to ``--find-renames``, if any.
+
+            **kwargs (dict, unused):
+                Unused keyword arguments.
+
         Returns:
             dict:
             A dictionary containing the following keys:
@@ -635,20 +696,24 @@ class GitClient(SCMClient):
         except KeyError:
             merge_base = revisions['base']
 
-        diff_lines = self.make_diff(merge_base,
-                                    revisions['base'],
-                                    revisions['tip'],
-                                    include_files,
-                                    exclude_patterns,
-                                    no_renames)
+        diff_lines = self.make_diff(
+            merge_base,
+            revisions['base'],
+            revisions['tip'],
+            include_files,
+            exclude_patterns,
+            no_renames,
+            find_renames_threshold=git_find_renames_threshold)
 
-        if 'parent_base' in revisions:
-            parent_diff_lines = self.make_diff(merge_base,
-                                               revisions['parent_base'],
-                                               revisions['base'],
-                                               include_files,
-                                               exclude_patterns,
-                                               no_renames)
+        if 'parent_base' in revisions and with_parent_diff:
+            parent_diff_lines = self.make_diff(
+                merge_base,
+                revisions['parent_base'],
+                revisions['base'],
+                include_files,
+                exclude_patterns,
+                no_renames,
+                find_renames_threshold=git_find_renames_threshold)
 
             base_commit_id = revisions['parent_base']
         else:
@@ -663,7 +728,7 @@ class GitClient(SCMClient):
         }
 
     def make_diff(self, merge_base, base, tip, include_files,
-                  exclude_patterns, no_renames=False):
+                  exclude_patterns, no_renames, find_renames_threshold):
         """Perform a diff on a particular branch range.
 
         Args:
@@ -684,6 +749,12 @@ class GitClient(SCMClient):
                 A list of shell-style glob patterns to blacklist during diff
                 generation.
 
+            no_renames (bool, optional):
+                Whether to skip rename detection.
+
+            find_renames_threshold (unicode, optional):
+                The threshhold to pass to ``--find-renames``, if any.
+
         Returns:
             bytes:
             The diff between (base, tip].
@@ -710,7 +781,12 @@ class GitClient(SCMClient):
             if (not no_renames and
                 self.capabilities is not None and
                 self.capabilities.has_capability('diffs', 'moved_files')):
-                diff_cmd_params.append('-M')
+
+                if find_renames_threshold is not None:
+                    diff_cmd_params.append('--find-renames=%s'
+                                           % find_renames_threshold)
+                else:
+                    diff_cmd_params.append('--find-renames')
             else:
                 diff_cmd_params.append('--no-renames')
         else:
@@ -1022,19 +1098,22 @@ class GitClient(SCMClient):
 
         cmd.append(patch_file)
 
-        rc, data = self._execute(cmd, ignore_errors=True, with_errors=True,
-                                 return_error_code=True)
+        rc, data = self._execute(cmd,
+                                 ignore_errors=True,
+                                 with_errors=True,
+                                 return_error_code=True,
+                                 results_unicode=False)
 
         if rc == 0:
             return PatchResult(applied=True, patch_output=data)
-        elif 'with conflicts' in data:
+        elif b'with conflicts' in data:
             return PatchResult(
                 applied=True,
                 has_conflicts=True,
                 conflicting_files=[
-                    line.split(' ', 1)[1]
+                    line.split(b' ', 1)[1]
                     for line in data.splitlines()
-                    if line.startswith('U')
+                    if line.startswith(b'U')
                 ],
                 patch_output=data)
         else:
@@ -1260,6 +1339,106 @@ class GitClient(SCMClient):
         """
         return execute(cmdline, cwd=self._git_toplevel, *args, **kwargs)
 
+    def get_commit_history(self, revisions):
+        """Return the commit history specified by the revisions.
+
+        Args:
+            revisions (dict):
+                A dictionary of revisions to generate history for, as returned
+                by :py:meth:`parse_revision_spec`.
+
+        Returns:
+            list of dict:
+            The list of history entries, in order. The dictionaries have the
+            following keys:
+
+            ``commit_id``:
+                The unique identifier of the commit.
+
+            ``parent_id``:
+                The unique identifier of the parent commit.
+
+            ``author_name``:
+                The name of the commit's author.
+
+            ``author_email``:
+                The e-mail address of the commit's author.
+
+            ``author_date``:
+                The date the commit was authored.
+
+            ``committer_name``:
+                The committer's name.
+
+            ``committer_email``:
+                The e-mail address of the committer.
+
+            ``committer_date``:
+                The date the commit was committed.
+
+            ``commit_message``:
+                The commit's message.
+
+        Raises:
+            rbtools.clients.errors.SCMError:
+                The history is non-linear or there is a commit with no parents.
+        """
+        log_fields = {
+            'commit_id': b'%H',
+            'parent_id': b'%P',
+            'author_name': b'%an',
+            'author_email': b'%ae',
+            'author_date': b'%ad',
+            'committer_name': b'%cn',
+            'committer_email': b'%ce',
+            'committer_date': b'%cd',
+            'commit_message': b'%B',
+        }
+
+        # 0x1f is the ASCII field separator. It is a non-printable character
+        # that should not appear in any field in `git log`.
+        log_format = b'%x1f'.join(six.itervalues(log_fields))
+
+        log_entries = execute(
+            [
+                self.git,
+                b'log',
+                b'-z',
+                b'--reverse',
+                b'--pretty=format:%s' % log_format,
+                b'--date=iso8601-strict',
+                b'%s..%s' % (revisions['base'].encode('utf-8'),
+                             revisions['tip'].encode('utf-8')),
+            ],
+            ignore_errors=True,
+            none_on_ignored_error=True,
+            results_unicode=True)
+
+        if not log_entries:
+            return None
+
+        history = []
+        field_names = six.viewkeys(log_fields)
+
+        for log_entry in log_entries.split(self._NUL):
+            fields = log_entry.split(self._FIELD_SEP)
+            entry = dict(zip(field_names, fields))
+
+            parents = entry['parent_id'].split()
+
+            if len(parents) > 1:
+                raise SCMError(
+                    'The Git SCMClient only supports posting commit histories '
+                    'that are entirely linear.')
+            elif len(parents) == 0:
+                raise SCMError(
+                    'The Git SCMClient only supports posting commits that '
+                    'have exactly one parent.')
+
+            history.append(entry)
+
+        return history
+
     def _get_remote(self, local_branch):
         """Return the remote for a given branch.
 
