@@ -51,6 +51,8 @@ class MercurialClient(SCMClient):
     """
 
     name = 'Mercurial'
+
+    supports_commit_history = True
     supports_diff_exclude_patterns = True
     can_bookmark = True
     can_branch = True
@@ -58,8 +60,28 @@ class MercurialClient(SCMClient):
 
     PRE_CREATION = '/dev/null'
     PRE_CREATION_DATE = 'Thu Jan 01 00:00:00 1970 +0000'
+    NO_PARENT = '0' * 40
 
-    def __init__(self, **kwargs):
+    # The ASCII field seperator.
+    _FIELD_SEP = '\x1f'
+
+    # The ASCII field separator as an escape sequence.
+    #
+    # This is passed to Mercurial, where it is interpreted and transformed into
+    # the actual character.
+    _FIELD_SEP_ESC = r'\x1f'
+
+    # The ASCII record separator.
+    _RECORD_SEP = '\x1e'
+
+    # The ASCII record separator as an escape sequence.
+    #
+    # This is passed to Mercurial, where it is interpreted and transformed into
+    # the actual character.
+    _RECORD_SEP_ESC = r'\x1e'
+
+
+    def __init__(self, executable='hg', **kwargs):
         """Initialize the client.
 
         Args:
@@ -69,6 +91,7 @@ class MercurialClient(SCMClient):
         super(MercurialClient, self).__init__(**kwargs)
 
         self.hgrc = {}
+        self._exe = executable
         self._type = 'hg'
         self._remote_path = ()
         self._initted = False
@@ -103,7 +126,7 @@ class MercurialClient(SCMClient):
         if not hasattr(self, '_hidden_changesets_supported'):
             # The choice of command is arbitrary. parents for the initial
             # revision should be fast.
-            result = execute(['hg', 'parents', '--hidden', '-r', '0'],
+            result = execute([self._exe, 'parents', '--hidden', '-r', '0'],
                              ignore_errors=True,
                              with_errors=False,
                              none_on_ignored_error=True)
@@ -120,11 +143,11 @@ class MercurialClient(SCMClient):
         returns None.
         """
         if not hasattr(self, '_hg_root'):
-            root = execute(['hg', 'root'], env=self._hg_env,
-                           ignore_errors=True)
+            self._load_hgrc()
 
-            if not root.startswith('abort:'):
-                self._hg_root = root.strip()
+            key = 'bundle.mainreporoot'
+            if key in self.hgrc:
+                self._hg_root = self.hgrc[key]
             else:
                 self._hg_root = None
 
@@ -135,11 +158,12 @@ class MercurialClient(SCMClient):
         if self._initted or not self.hg_root:
             return
 
-        self._load_hgrc()
+        if 'extensions.hgsubversion' in self.hgrc:
+            svn_info = execute([self._exe, 'svn', 'info'], ignore_errors=True)
+        else:
+            svn_info = None
 
-        svn_info = execute(['hg', 'svn', 'info'], ignore_errors=True)
-
-        if (not svn_info.startswith('abort:') and
+        if (svn_info and not svn_info.startswith('abort:') and
             not svn_info.startswith('hg: unknown command') and
             not svn_info.lower().startswith('not a child of')):
             self._type = 'svn'
@@ -158,6 +182,78 @@ class MercurialClient(SCMClient):
 
         self._initted = True
 
+    def get_commit_history(self, revisions):
+        """Return the commit history specified by the revisions.
+
+        Args:
+            revisions (dict):
+                A dictionary of revisions to generate history for, as returned
+                by :py:meth:`parse_revision_spec`.
+
+        Returns:
+            list of dict:
+            This list of history entries, in order.
+
+        Raises:
+            rbtools.clients.errors.SCMError:
+                The history is non-linear or there is a commit with no parents.
+        """
+        log_fields = {
+            'commit_id': '{node}',
+            'parent_id': '{p1node}',
+            'author_name': '{author|person}',
+            'author_email': '{author|email}',
+            'author_date': '{date|rfc3339date}',
+            'parent2': '{p2node}',
+            'commit_message': '{desc}',
+        }
+        log_format = self._FIELD_SEP_ESC.join(six.itervalues(log_fields))
+
+        log_entries = execute(
+            [
+                self._exe,
+                'log',
+                '--template',
+                '%s%s' % (log_format, self._RECORD_SEP_ESC),
+                '-r',
+                '%(base)s::%(tip)s and not %(base)s' % revisions,
+            ],
+            ignore_errors=True,
+            none_on_ignored_error=True,
+            results_unicode=True)
+
+        if not log_entries:
+            return None
+
+        history = []
+        field_names = six.viewkeys(log_fields)
+
+        # The ASCII record separator will be appended to every record, so if we
+        # attempt to split the entire output by the record separator, we will
+        # end up with an empty ``log_entry`` at the end, which will cause
+        # errors.
+        for log_entry in log_entries[:-1].split(self._RECORD_SEP):
+            fields = log_entry.split(self._FIELD_SEP)
+            entry = dict(zip(field_names, fields))
+
+            # We do not want `parent2` to be included in the entry because
+            # the entry's items are used as the keyword arguments to the
+            # method that uploads a commit and it would be unexpected.
+            if entry.pop('parent2') != self.NO_PARENT:
+                raise SCMError(
+                    'The Mercurial SCMClient only supports posting commit '
+                    'histories that are entirely linear.'
+                )
+            elif entry['parent_id'] == self.NO_PARENT:
+                raise SCMError(
+                    'The Mercurial SCMClient only supports posting commits '
+                    'that have exactly one parent.'
+                )
+
+            history.append(entry)
+
+        return history
+
     def get_repository_info(self):
         """Return the repository info object.
 
@@ -165,7 +261,7 @@ class MercurialClient(SCMClient):
             rbtools.clients.RepositoryInfo:
             The repository info structure.
         """
-        if not check_install(['hg', '--help']):
+        if not check_install([self._exe, '--help']):
             logging.debug('Unable to execute "hg --help": skipping Mercurial')
             return None
 
@@ -253,6 +349,7 @@ class MercurialClient(SCMClient):
             n_revisions = len(revisions)
 
         result = {}
+
         if n_revisions == 0:
             # No revisions: Find the outgoing changes. Only consider the
             # working copy revision and ancestors because that makes sense.
@@ -316,7 +413,7 @@ class MercurialClient(SCMClient):
             result['tip'] = self._identify_revision(revisions[0])
             result['commit_id'] = result['tip']
             result['base'] = self._execute(
-                ['hg', 'parents', '--hidden', '-r', result['tip'],
+                [self._exe, 'parents', '--hidden', '-r', result['tip'],
                  '--template', '{node|short}']).split()[0]
             if len(result['base']) != 12:
                 raise InvalidRevisionSpecError(
@@ -346,7 +443,7 @@ class MercurialClient(SCMClient):
                 return result
 
             parent_base = self._execute(
-                ['hg', 'parents', '--hidden', '-r', outgoing[0][1],
+                [self._exe, 'parents', '--hidden', '-r', outgoing[0][1],
                  '--template', '{node|short}']).split()
 
             if len(parent_base) == 0:
@@ -377,7 +474,7 @@ class MercurialClient(SCMClient):
             The global revision ID of the commit.
         """
         identify = self._execute(
-            ['hg', 'identify', '-i', '--hidden', '-r', str(revision)],
+            [self._exe, 'identify', '-i', '--hidden', '-r', str(revision)],
             ignore_errors=True, none_on_ignored_error=True)
 
         if identify is None:
@@ -425,7 +522,8 @@ class MercurialClient(SCMClient):
 
     def _load_hgrc(self):
         """Load the hgrc file."""
-        for line in execute(['hg', 'showconfig'], split_lines=True):
+        for line in execute([self._exe, 'showconfig'],
+                            env=self._hg_env, split_lines=True):
             line = line.split('=', 1)
 
             if len(line) == 2:
@@ -452,7 +550,7 @@ class MercurialClient(SCMClient):
             :py:class:`MercurialRefType`.
         """
         # Check for any bookmarks matching ref.
-        rc, output = self._execute(['hg', 'log', '-ql1', '-r',
+        rc, output = self._execute([self._exe, 'log', '-ql1', '-r',
                                     'bookmark(%s)' % ref],
                                    ignore_errors=True,
                                    return_error_code=True)
@@ -465,13 +563,13 @@ class MercurialClient(SCMClient):
         # Ideally, we'd use the same sort of log call we'd use for bookmarks
         # and tags, but it works differently for branches, and will
         # incorrectly match tags.
-        branches = self._execute(['hg', 'branches', '-q']).split()
+        branches = self._execute([self._exe, 'branches', '-q']).split()
 
         if ref in branches:
             return MercurialRefType.BRANCH
 
         # Check for any tags matching ref.
-        rc, output = self._execute(['hg', 'log', '-ql1', '-r',
+        rc, output = self._execute([self._exe, 'log', '-ql1', '-r',
                                     'tag(%s)' % ref],
                                    ignore_errors=True,
                                    return_error_code=True)
@@ -480,7 +578,7 @@ class MercurialClient(SCMClient):
             return MercurialRefType.TAG
 
         # Now just check that it exists at all. We'll assume it's a revision.
-        rc, output = self._execute(['hg', 'identify', '-r', ref],
+        rc, output = self._execute([self._exe, 'identify', '-r', ref],
                                    ignore_errors=True,
                                    return_error_code=True)
 
@@ -508,7 +606,7 @@ class MercurialClient(SCMClient):
 
         delim = str(uuid.uuid1())
         descs = self._execute(
-            ['hg', 'log', '--hidden', '-r', '%s::%s' % (rev1, rev2),
+            [self._exe, 'log', '--hidden', '-r', '%s::%s' % (rev1, rev2),
              '--template', '{desc}%s' % delim],
             env=self._hg_env)
 
@@ -520,9 +618,14 @@ class MercurialClient(SCMClient):
         return '\n\n'.join(desc.strip() for desc in descs)
 
     def diff(self, revisions, include_files=[], exclude_patterns=[],
-             no_renames=False, extra_args=[]):
+             extra_args=[], with_parent_diff=True, **kwargs):
         """Perform a diff using the given revisions.
 
+        This will generate a Git-style diff and parent diff (if needed) for
+        the provided revisions. The diff will contain additional metadata
+        headers used by Review Board to locate the appropriate revisions from
+        the repository.
+
         Args:
             revisions (dict):
                 A dictionary of revisions, as returned by
@@ -539,6 +642,12 @@ class MercurialClient(SCMClient):
                 Additional arguments to be passed to the diff generation.
                 Unused for mercurial.
 
+            with_parent_diff (bool, optional):
+                Whether or not to include the parent diff in the result.
+
+            **kwargs (dict, unused):
+                Unused keyword arguments.
+
         Returns:
             dict:
             A dictionary containing the following keys:
@@ -559,39 +668,41 @@ class MercurialClient(SCMClient):
         """
         self._init()
 
-        diff_cmd = ['hg', 'diff', '--hidden', '--nodates']
-
-        if self.supports_empty_files():
-            diff_cmd.append('-g')
+        diff_args = ['--hidden', '--nodates', '-g']
 
         if self._type == 'svn':
-            diff_cmd.append('--svn')
+            diff_args.append('--svn')
 
-        diff_cmd += include_files
+        diff_args += include_files
 
         for pattern in exclude_patterns:
-            diff_cmd.append('-X')
-            diff_cmd.append(pattern)
+            diff_args += ['-X', pattern]
 
-        diff = self._execute(
-            diff_cmd + ['-r', revisions['base'], '-r', revisions['tip']],
-            env=self._hg_env, log_output_on_error=False, results_unicode=False)
+        node_base_id = revisions['base']
 
-        if 'parent_base' in revisions:
+        diff = self._run_diff(diff_args,
+                              parent_id=node_base_id,
+                              node_id=revisions['tip'])
+
+        if with_parent_diff and 'parent_base' in revisions:
             base_commit_id = revisions['parent_base']
-            parent_diff = self._execute(
-                diff_cmd + ['-r', base_commit_id, '-r', revisions['base']],
-                env=self._hg_env, results_unicode=False)
+            parent_diff = self._run_diff(diff_args,
+                                         parent_id=base_commit_id,
+                                         node_id=node_base_id)
         else:
-            base_commit_id = revisions['base']
+            base_commit_id = node_base_id
             parent_diff = None
 
         # If reviewboard requests a relative revision via hgweb it will fail
         # since hgweb does not support the relative revision syntax (^1, -1).
         # Rewrite this relative node id to an absolute node id.
-        base_commit_id = self._execute(
-            ['hg', 'log', '-r', base_commit_id, '--template', '{node}'],
-            env=self._hg_env, results_unicode=False)
+        match = re.match(r'^[a-z|A-Z|0-9]*$', base_commit_id)
+
+        if not match:
+            base_commit_id = self._execute(
+                [self._exe, 'log', '-r', base_commit_id,
+                 '--template', '{node}'],
+                env=self._hg_env, results_unicode=False)
 
         return {
             'diff': diff,
@@ -600,6 +711,76 @@ class MercurialClient(SCMClient):
             'base_commit_id': base_commit_id,
         }
 
+    def _run_diff(self, diff_args, parent_id, node_id):
+        """Run a diff command and normalize its results.
+
+        This will run :command:`hg diff` with the provided arguments for the
+        provided revision range, performing some normalization on the diff to
+        prepare it for use in Review Board.
+
+        Args:
+            diff_args (list of unicode):
+                The arguments to pass to :command:`hg diff` (except for any
+                revision ranges).
+
+            parent_id (unicode):
+                The ID of the parent commit for the range.
+
+            node_id (unicode):
+                The ID of the latest commit for the range.
+
+        Returns:
+            bytes:
+            The normalized diff content.
+        """
+        diff = self._execute(
+            [self._exe, 'diff'] + diff_args + ['-r', parent_id, '-r', node_id],
+            env=self._hg_env,
+            log_output_on_error=False,
+            results_unicode=False)
+
+        return self._normalize_diff(diff,
+                                    node_id=node_id,
+                                    parent_id=parent_id)
+
+    def _normalize_diff(self, diff, node_id, parent_id):
+        """Normalize a diff, adding any headers that may be needed.
+
+        For Git-style diffs, this will ensure the diff starts with information
+        required for Review Board to identify the commit and its parent. These
+        are based on headers normally generated by :command:`hg export`.
+
+        Args:
+            diff (bytes):
+                The generated diff content to prepend to.
+
+            node_id (unicode):
+                The revision of this change.
+
+            parent_id (unicode):
+                The revision of the parent change.
+
+        Returns:
+            bytes:
+            The normalized diff content.
+        """
+        assert isinstance(diff, bytes)
+
+        if diff.lstrip().startswith(b'diff --git'):
+            diff = (
+                b'# HG changeset patch\n'
+                b'# Node ID %(node_id)s\n'
+                b'# Parent  %(parent_id)s\n'
+                b'%(diff)s'
+                % {
+                    b'node_id': node_id.encode('utf-8'),
+                    b'parent_id': parent_id.encode('utf-8'),
+                    b'diff': diff,
+                }
+            )
+
+        return diff
+
     def _get_files_in_changeset(self, rev):
         """Return a set of all files in the specified changeset.
 
@@ -611,7 +792,7 @@ class MercurialClient(SCMClient):
             set:
             A set of filenames in the changeset.
         """
-        cmd = ['hg', 'locate', '-r', rev]
+        cmd = [self._exe, 'locate', '-r', rev]
 
         files = execute(cmd, env=self._hg_env, ignore_errors=True,
                         none_on_ignored_error=True)
@@ -635,7 +816,7 @@ class MercurialClient(SCMClient):
             The branch branch for the hgsubversion checkout.
         """
         return (getattr(self.options, 'tracking', None) or
-                execute(['hg', 'parent', '--svn', '--template',
+                execute([self._exe, 'parent', '--svn', '--template',
                         '{node}\n']).strip())
 
     def _get_remote_branch(self):
@@ -717,7 +898,7 @@ class MercurialClient(SCMClient):
                 "A commit message wasn't provided. The patched files are in "
                 "your tree but haven't been committed.")
 
-        hg_command = ['hg', 'commit', '-m', modified_message]
+        hg_command = [self._exe, 'commit', '-m', modified_message]
 
         try:
             hg_command += ['-u', '%s <%s>' % (author.fullname, author.email)]
@@ -784,26 +965,26 @@ class MercurialClient(SCMClient):
 
         if close_branch and ref_type == MercurialRefType.BRANCH:
             try:
-                self._execute(['hg', 'update', target])
+                self._execute([self._exe, 'update', target])
             except Exception as e:
                 raise MergeError('Could not switch to branch "%s".\n\n%s'
                                  % (target, e))
 
             try:
-                self._execute(['hg', 'commit', '-m', message,
+                self._execute([self._exe, 'commit', '-m', message,
                                '--close-branch'])
             except Exception as e:
                 raise MergeError('Could not close branch "%s".\n\n%s'
                                  % (target, e))
 
         try:
-            self._execute(['hg', 'update', destination])
+            self._execute([self._exe, 'update', destination])
         except Exception as e:
             raise MergeError('Could not switch to branch "%s".\n\n%s'
                              % (destination, e))
 
         try:
-            self._execute(['hg', 'merge', target])
+            self._execute([self._exe, 'merge', target])
         except Exception as e:
             raise MergeError('Could not merge %s "%s" into "%s".\n\n%s'
                              % (ref_type, target, destination, e))
@@ -814,7 +995,7 @@ class MercurialClient(SCMClient):
 
         if close_branch and ref_type == MercurialRefType.BOOKMARK:
             try:
-                self._execute(['hg', 'bookmark', '-d', target])
+                self._execute([self._exe, 'bookmark', '-d', target])
             except Exception as e:
                 raise MergeError('Could not delete bookmark "%s".\n\n%s'
                                  % (target, e))
@@ -826,7 +1007,7 @@ class MercurialClient(SCMClient):
             unicode:
             The name of the currently checked-out branch.
         """
-        return execute(['hg', 'branch'], env=self._hg_env).strip()
+        return execute([self._exe, 'branch'], env=self._hg_env).strip()
 
     def _get_bottom_and_top_outgoing_revs_for_remote(self, rev=None):
         """Return the bottom and top outgoing revisions.
@@ -875,7 +1056,7 @@ class MercurialClient(SCMClient):
             outgoing changeset. The list will be sorted in revision order.
         """
         outgoing_changesets = []
-        args = ['hg', '-q', 'outgoing', '--template',
+        args = [self._exe, '-q', 'outgoing', '--template',
                 '{rev}\\t{node|short}\\t{branch}\\n',
                 remote]
         if rev:
@@ -927,7 +1108,7 @@ class MercurialClient(SCMClient):
 
         for rev, node, branch in reversed(outgoing_changesets):
             parents = execute(
-                ['hg', 'log', '-r', str(rev), '--template', '{parents}'],
+                [self._exe, 'log', '-r', str(rev), '--template', '{parents}'],
                 env=self._hg_env)
             parents = re.split(':[^\s]+\s*', parents)
             parents = [int(p) for p in parents if p != '']
@@ -1017,7 +1198,7 @@ class MercurialClient(SCMClient):
             ``True`` if the working directory has been modified, otherwise
             returns ``False``.
         """
-        status = execute(['hg', 'status', '--modified', '--added',
+        status = execute([self._exe, 'status', '--modified', '--added',
                           '--removed', '--deleted'])
         return status != ''
 
@@ -1051,14 +1232,15 @@ class MercurialClient(SCMClient):
             rbtools.clients.PatchResult:
             The result of the patch operation.
         """
-        cmd = ['hg', 'patch', '--no-commit']
+        cmd = [self._exe, 'patch', '--no-commit']
 
         if p:
             cmd += ['-p', p]
 
         cmd.append(patch_file)
 
-        rc, data = self._execute(cmd, with_errors=True, return_error_code=True)
+        rc, data = self._execute(cmd, with_errors=True, return_error_code=True,
+                                 results_unicode=False)
 
         return PatchResult(applied=(rc == 0), patch_output=data)
 
@@ -1096,8 +1278,8 @@ class MercurialClient(SCMClient):
         if added_files:
             added_files = self._strip_p_num_slashes(added_files, int(p_num))
             make_empty_files(added_files)
-            result = execute(['hg', 'add'] + added_files, ignore_errors=True,
-                             none_on_ignored_error=True)
+            result = execute([self._exe, 'add'] + added_files,
+                             ignore_errors=True, none_on_ignored_error=True)
 
             if result is None:
                 logging.error('Unable to execute "hg add" on: %s',
@@ -1108,7 +1290,7 @@ class MercurialClient(SCMClient):
         if deleted_files:
             deleted_files = self._strip_p_num_slashes(deleted_files,
                                                       int(p_num))
-            result = execute(['hg', 'remove'] + deleted_files,
+            result = execute([self._exe, 'remove'] + deleted_files,
                              ignore_errors=True, none_on_ignored_error=True)
 
             if result is None:
@@ -1137,4 +1319,4 @@ class MercurialClient(SCMClient):
             unicode:
             A string with the name of the current bookmark.
         """
-        return execute(['hg', 'id', '-B'], ignore_errors=True).strip()
+        return execute([self._exe, 'id', '-B'], ignore_errors=True).strip()
