@@ -137,14 +137,143 @@ class PerforceClientTests(SCMClientTests):
 
         self.assertEqual(url, RB_URL)
 
-    def test_repository_info(self):
-        """Testing PerforceClient.get_repository_info"""
+    def test_repository_info_with_server_address(self):
+        """Testing PerforceClient.get_repository_info with server address"""
         SERVER_PATH = 'perforce.example.com:1666'
 
         class TestWrapper(P4Wrapper):
             def is_supported(self):
                 return True
 
+            def counters(self):
+                return {}
+
+            def info(self):
+                return {
+                    'Client root': os.getcwd(),
+                    'Server address': SERVER_PATH,
+                    'Server version': 'P4D/FREEBSD60X86_64/2012.2/525804 '
+                                      '(2012/09/18)',
+                }
+
+        client = PerforceClient(TestWrapper)
+        info = client.get_repository_info()
+
+        self.assertIsNotNone(info)
+        self.assertEqual(info.path, SERVER_PATH)
+        self.assertIsNone(info.name)
+        self.assertEqual(client.p4d_version, (2012, 2))
+
+    def test_repository_info_with_broker_address(self):
+        """Testing PerforceClient.get_repository_info with broker address"""
+        BROKER_PATH = 'broker.example.com:1666'
+        SERVER_PATH = 'perforce.example.com:1666'
+
+        class TestWrapper(P4Wrapper):
+            def is_supported(self):
+                return True
+
+            def counters(self):
+                return {}
+
+            def info(self):
+                return {
+                    'Client root': os.getcwd(),
+                    'Broker address': BROKER_PATH,
+                    'Server address': SERVER_PATH,
+                    'Server version': 'P4D/FREEBSD60X86_64/2012.2/525804 '
+                                      '(2012/09/18)',
+                }
+
+        client = PerforceClient(TestWrapper)
+        info = client.get_repository_info()
+
+        self.assertIsNotNone(info)
+        self.assertEqual(info.path, BROKER_PATH)
+        self.assertIsNone(info.name)
+        self.assertEqual(client.p4d_version, (2012, 2))
+
+    def test_repository_info_with_server_address_and_encrypted(self):
+        """Testing PerforceClient.get_repository_info with server address
+        and broker encryption"""
+        SERVER_PATH = 'perforce.example.com:1666'
+
+        class TestWrapper(P4Wrapper):
+            def is_supported(self):
+                return True
+
+            def counters(self):
+                return {}
+
+            def info(self):
+                return {
+                    'Client root': os.getcwd(),
+                    'Server address': SERVER_PATH,
+                    'Server encryption': 'encrypted',
+                    'Server version': 'P4D/FREEBSD60X86_64/2012.2/525804 '
+                                      '(2012/09/18)',
+                }
+
+        client = PerforceClient(TestWrapper)
+        info = client.get_repository_info()
+
+        self.assertIsNotNone(info)
+        self.assertEqual(info.path, [
+            'ssl:%s' % SERVER_PATH,
+            SERVER_PATH,
+        ])
+        self.assertIsNone(info.name)
+        self.assertEqual(client.p4d_version, (2012, 2))
+
+    def test_repository_info_with_broker_address_and_encrypted(self):
+        """Testing PerforceClient.get_repository_info with broker address
+        and broker encryption"""
+        BROKER_PATH = 'broker.example.com:1666'
+        SERVER_PATH = 'perforce.example.com:1666'
+
+        class TestWrapper(P4Wrapper):
+            def is_supported(self):
+                return True
+
+            def counters(self):
+                return {}
+
+            def info(self):
+                return {
+                    'Client root': os.getcwd(),
+                    'Broker address': BROKER_PATH,
+                    'Broker encryption': 'encrypted',
+                    'Server address': SERVER_PATH,
+                    'Server version': 'P4D/FREEBSD60X86_64/2012.2/525804 '
+                                      '(2012/09/18)',
+                }
+
+        client = PerforceClient(TestWrapper)
+        info = client.get_repository_info()
+
+        self.assertIsNotNone(info)
+        self.assertEqual(info.path, [
+            'ssl:%s' % BROKER_PATH,
+            BROKER_PATH,
+        ])
+        self.assertIsNone(info.name)
+        self.assertEqual(client.p4d_version, (2012, 2))
+
+    def test_repository_info_with_repository_name_counter(self):
+        """Testing PerforceClient.get_repository_info with repository name
+        counter
+        """
+        SERVER_PATH = 'perforce.example.com:1666'
+
+        class TestWrapper(P4Wrapper):
+            def is_supported(self):
+                return True
+
+            def counters(self):
+                return {
+                    'reviewboard.repository_name': 'myrepo',
+                }
+
             def info(self):
                 return {
                     'Client root': os.getcwd(),
@@ -158,6 +287,7 @@ class PerforceClientTests(SCMClientTests):
 
         self.assertNotEqual(info, None)
         self.assertEqual(info.path, SERVER_PATH)
+        self.assertEqual(info.name, 'myrepo')
         self.assertEqual(client.p4d_version, (2012, 2))
 
     def test_repository_info_outside_client_root(self):
