@@ -9,12 +9,14 @@ import sys
 
 import colorama
 import pkg_resources
+import six
 from six.moves.urllib.parse import urlparse
 
 from rbtools import get_version_string
 from rbtools.api.capabilities import Capabilities
 from rbtools.api.client import RBClient
 from rbtools.api.errors import APIError, ServerInterfaceError
+from rbtools.api.transport.sync import SyncTransport
 from rbtools.clients import scan_usable_client
 from rbtools.clients.errors import OptionsCheckError
 from rbtools.utils.console import get_input, get_pass
@@ -272,6 +274,24 @@ class Command(object):
                    default=None,
                    help='The file to use for the API cache database.',
                    added_in='0.7.3'),
+            Option('--ca-certs',
+                   dest='ca_certs',
+                   metavar='FILE',
+                   config_key='CA_CERTS',
+                   default=None,
+                   help='Additional TLS CA bundle.'),
+            Option('--client-key',
+                   dest='client_key',
+                   metavar='FILE',
+                   config_key='CLIENT_KEY',
+                   default=None,
+                   help='Key for TLS client authentication.'),
+            Option('--client-cert',
+                   dest='client_cert',
+                   metavar='FILE',
+                   config_key='CLIENT_CERT',
+                   default=None,
+                   help='Certificate for TLS client authentication.'),
         ]
     )
 
@@ -429,6 +449,20 @@ class Command(object):
         ]
     )
 
+    git_options = OptionGroup(
+        name='Git Options',
+        description='Git-specific options for diff generation.',
+        option_list=[
+            Option('--git-find-renames-threshold',
+                   dest='git_find_renames_threshold',
+                   metavar='THRESHOLD',
+                   default=None,
+                   help='The threshold to pass to `--find-renames` when '
+                        'generating a git diff.'
+                        '\n'
+                        'For more information, see `git help diff`.'),
+        ])
+
     perforce_options = OptionGroup(
         name='Perforce Options',
         description='Perforce-specific options for selecting the '
@@ -538,8 +572,19 @@ class Command(object):
         ]
     )
 
-    def __init__(self):
+    default_transport_cls = SyncTransport
+
+    def __init__(self, transport_cls=SyncTransport):
+        """Initialize the base functionality for the command.
+
+        Args:
+            transport_cls (rbtools.api.transport.Transport, optional):
+                The transport class used for all API communication. By default,
+                this uses the transport defined in
+                :py:attr:`default_transport_cls`.
+        """
         self.log = logging.getLogger('rb.%s' % self.name)
+        self.transport_cls = transport_cls or self.default_transport_cls
 
     def create_parser(self, config, argv=[]):
         """Create and return the argument parser for this command."""
@@ -747,16 +792,37 @@ class Command(object):
         cleanup_tempfiles()
         sys.exit(exit_code)
 
-    def initialize_scm_tool(self, client_name=None):
-        """Initialize the SCM tool for the current working directory."""
-        repository_info, tool = scan_usable_client(self.config,
-                                                   self.options,
-                                                   client_name=client_name)
+    def initialize_scm_tool(self, client_name=None,
+                            require_repository_info=True):
+        """Initialize the SCM tool for the current working directory.
+
+        Args:
+            client_name (unicode, optional):
+                A specific client name, which can come from the configuration.
+                This can be used to disambiguate if there are nested
+                repositories, or to speed up detection.
+
+            require_repository_info (bool, optional):
+                Whether information on a repository is required. This is the
+                default. If disabled, this will return ``None`` for the
+                repository information if a matching repository could not be
+                found.
+
+        Returns:
+            tuple:
+            A 2-tuple, containing the repository info structure and the tool
+            instance.
+        """
+        repository_info, tool = scan_usable_client(
+            self.config,
+            self.options,
+            client_name=client_name,
+            require_repository_info=require_repository_info)
 
         try:
             tool.check_options()
         except OptionsCheckError as e:
-            raise CommandError('%s\n' % e)
+            raise CommandError(six.text_type(e))
 
         return repository_info, tool
 
@@ -784,7 +850,7 @@ class Command(object):
         """
         if self.options.server:
             server_url = self.options.server
-        elif tool:
+        elif tool and repository_info is not None:
             server_url = tool.scan_for_server(repository_info)
         else:
             server_url = None
@@ -887,7 +953,11 @@ class Command(object):
             cache_location=self.options.cache_location,
             in_memory_cache=self.options.in_memory_cache,
             save_cookies=self.options.save_cookies,
-            ext_auth_cookies=self.options.ext_auth_cookies)
+            ext_auth_cookies=self.options.ext_auth_cookies,
+            ca_certs=self.options.ca_certs,
+            client_key=self.options.client_key,
+            client_cert=self.options.client_cert,
+            transport_cls=self.transport_cls)
 
     def get_api(self, server_url):
         """Returns an RBClient instance and the associated root resource.
